/* -*- C++ -*-
 *
 * Copyright (C) 2002 Jean-Daniel Fekete
 * 
 * This file is part of MillionVis.
 * 
 * MillionVis is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 * 
 * MillionVis is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with MillionVis; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA. 
 */
#ifndef INFOVIS_DRAWING_KEYCODES_HPP
#define INFOVIS_DRAWING_KEYCODES_HPP

namespace infovis {

enum KeyCode {
  keycode_none,
  keycode_backspace			= 0x08,
  keycode_tab				= 0x09,
  keycode_linefeed			= 0x0A,
  keycode_return			= 0x0D,
  keycode_escape			= 0x1B,
  keycode_space               		= 0x020,
  keycode_exclam              		= 0x021,
  keycode_quotedbl            		= 0x022,
  keycode_numbersign          		= 0x023,
  keycode_dollar              		= 0x024,
  keycode_percent             		= 0x025,
  keycode_ampersand           		= 0x026,
  keycode_apostrophe          		= 0x027,
  keycode_quoteright          		= 0x027	/* deprecated */,
  keycode_parenleft           		= 0x028,
  keycode_parenright          		= 0x029,
  keycode_asterisk            		= 0x02a,
  keycode_plus                		= 0x02b,
  keycode_comma               		= 0x02c,
  keycode_minus               		= 0x02d,
  keycode_period              		= 0x02e,
  keycode_slash               		= 0x02f,
  keycode_0                   		= 0x030,
  keycode_1                   		= 0x031,
  keycode_2                   		= 0x032,
  keycode_3                   		= 0x033,
  keycode_4                   		= 0x034,
  keycode_5                   		= 0x035,
  keycode_6                   		= 0x036,
  keycode_7                   		= 0x037,
  keycode_8                   		= 0x038,
  keycode_9                   		= 0x039,
  keycode_colon               		= 0x03a,
  keycode_semicolon           		= 0x03b,
  keycode_less                		= 0x03c,
  keycode_equal               		= 0x03d,
  keycode_greater             		= 0x03e,
  keycode_question            		= 0x03f,
  keycode_at                  		= 0x040,
  keycode_A                   		= 0x041,
  keycode_B                   		= 0x042,
  keycode_C                   		= 0x043,
  keycode_D                   		= 0x044,
  keycode_E                   		= 0x045,
  keycode_F                   		= 0x046,
  keycode_G                   		= 0x047,
  keycode_H                   		= 0x048,
  keycode_I                   		= 0x049,
  keycode_J                   		= 0x04a,
  keycode_K                   		= 0x04b,
  keycode_L                   		= 0x04c,
  keycode_M                   		= 0x04d,
  keycode_N                   		= 0x04e,
  keycode_O                   		= 0x04f,
  keycode_P                   		= 0x050,
  keycode_Q                   		= 0x051,
  keycode_R                   		= 0x052,
  keycode_S                   		= 0x053,
  keycode_T                   		= 0x054,
  keycode_U                   		= 0x055,
  keycode_V                   		= 0x056,
  keycode_W                   		= 0x057,
  keycode_X                   		= 0x058,
  keycode_Y                   		= 0x059,
  keycode_Z                   		= 0x05a,
  keycode_bracketleft         		= 0x05b,
  keycode_backslash           		= 0x05c,
  keycode_bracketright        		= 0x05d,
  keycode_asciicircum         		= 0x05e,
  keycode_underscore          		= 0x05f,
  keycode_grave               		= 0x060,
  keycode_quoteleft           		= 0x060	/* deprecated */,
  keycode_a                   		= 0x061,
  keycode_b                   		= 0x062,
  keycode_c                   		= 0x063,
  keycode_d                   		= 0x064,
  keycode_e                   		= 0x065,
  keycode_f                   		= 0x066,
  keycode_g                   		= 0x067,
  keycode_h                   		= 0x068,
  keycode_i                   		= 0x069,
  keycode_j                   		= 0x06a,
  keycode_k                   		= 0x06b,
  keycode_l                   		= 0x06c,
  keycode_m                   		= 0x06d,
  keycode_n                   		= 0x06e,
  keycode_o                   		= 0x06f,
  keycode_p                   		= 0x070,
  keycode_q                   		= 0x071,
  keycode_r                   		= 0x072,
  keycode_s                   		= 0x073,
  keycode_t                   		= 0x074,
  keycode_u                   		= 0x075,
  keycode_v                   		= 0x076,
  keycode_w                   		= 0x077,
  keycode_x                   		= 0x078,
  keycode_y                   		= 0x079,
  keycode_z                   		= 0x07a,
  keycode_braceleft           		= 0x07b,
  keycode_bar                 		= 0x07c,
  keycode_braceright          		= 0x07d,
  keycode_asciitilde          		= 0x07e,

  keycode_nobreakspace        		= 0x0a0,
  keycode_exclamdown          		= 0x0a1,
  keycode_cent        	       		= 0x0a2,
  keycode_sterling            		= 0x0a3,
  keycode_currency            		= 0x0a4,
  keycode_yen                 		= 0x0a5,
  keycode_brokenbar           		= 0x0a6,
  keycode_section             		= 0x0a7,
  keycode_diaeresis           		= 0x0a8,
  keycode_copyright           		= 0x0a9,
  keycode_ordfeminine         		= 0x0aa,
  keycode_guillemotleft       		= 0x0ab	/* left angle quotation mark */,
  keycode_notsign             		= 0x0ac,
  keycode_hyphen              		= 0x0ad,
  keycode_registered          		= 0x0ae,
  keycode_macron              		= 0x0af,
  keycode_degree              		= 0x0b0,
  keycode_plusminus           		= 0x0b1,
  keycode_twosuperior         		= 0x0b2,
  keycode_threesuperior       		= 0x0b3,
  keycode_acute               		= 0x0b4,
  keycode_mu                  		= 0x0b5,
  keycode_paragraph           		= 0x0b6,
  keycode_periodcentered      		= 0x0b7,
  keycode_cedilla             		= 0x0b8,
  keycode_onesuperior         		= 0x0b9,
  keycode_masculine           		= 0x0ba,
  keycode_guillemotright      		= 0x0bb	/* right angle quotation mark */,
  keycode_onequarter          		= 0x0bc,
  keycode_onehalf             		= 0x0bd,
  keycode_threequarters       		= 0x0be,
  keycode_questiondown        		= 0x0bf,
  keycode_Agrave              		= 0x0c0,
  keycode_Aacute              		= 0x0c1,
  keycode_Acircumflex         		= 0x0c2,
  keycode_Atilde              		= 0x0c3,
  keycode_Adiaeresis          		= 0x0c4,
  keycode_Aring               		= 0x0c5,
  keycode_AE                  		= 0x0c6,
  keycode_Ccedilla            		= 0x0c7,
  keycode_Egrave              		= 0x0c8,
  keycode_Eacute              		= 0x0c9,
  keycode_Ecircumflex         		= 0x0ca,
  keycode_Ediaeresis          		= 0x0cb,
  keycode_Igrave              		= 0x0cc,
  keycode_Iacute              		= 0x0cd,
  keycode_Icircumflex         		= 0x0ce,
  keycode_Idiaeresis          		= 0x0cf,
  keycode_ETH                 		= 0x0d0,
  keycode_Eth                 		= 0x0d0	/* deprecated */,
  keycode_Ntilde              		= 0x0d1,
  keycode_Ograve              		= 0x0d2,
  keycode_Oacute              		= 0x0d3,
  keycode_Ocircumflex         		= 0x0d4,
  keycode_Otilde              		= 0x0d5,
  keycode_Odiaeresis          		= 0x0d6,
  keycode_multiply            		= 0x0d7,
  keycode_Ooblique            		= 0x0d8,
  keycode_Ugrave              		= 0x0d9,
  keycode_Uacute              		= 0x0da,
  keycode_Ucircumflex         		= 0x0db,
  keycode_Udiaeresis          		= 0x0dc,
  keycode_Yacute              		= 0x0dd,
  keycode_THORN               		= 0x0de,
  keycode_Thorn               		= 0x0de	/* deprecated */,
  keycode_ssharp              		= 0x0df,
  keycode_agrave              		= 0x0e0,
  keycode_aacute              		= 0x0e1,
  keycode_acircumflex         		= 0x0e2,
  keycode_atilde              		= 0x0e3,
  keycode_adiaeresis          		= 0x0e4,
  keycode_aring               		= 0x0e5,
  keycode_ae                  		= 0x0e6,
  keycode_ccedilla            		= 0x0e7,
  keycode_egrave              		= 0x0e8,
  keycode_eacute              		= 0x0e9,
  keycode_ecircumflex         		= 0x0ea,
  keycode_ediaeresis          		= 0x0eb,
  keycode_igrave              		= 0x0ec,
  keycode_iacute              		= 0x0ed,
  keycode_icircumflex         		= 0x0ee,
  keycode_idiaeresis          		= 0x0ef,
  keycode_eth                 		= 0x0f0,
  keycode_ntilde              		= 0x0f1,
  keycode_ograve              		= 0x0f2,
  keycode_oacute              		= 0x0f3,
  keycode_ocircumflex         		= 0x0f4,
  keycode_otilde              		= 0x0f5,
  keycode_odiaeresis          		= 0x0f6,
  keycode_division            		= 0x0f7,
  keycode_oslash              		= 0x0f8,
  keycode_ugrave              		= 0x0f9,
  keycode_uacute              		= 0x0fa,
  keycode_ucircumflex         		= 0x0fb,
  keycode_udiaeresis          		= 0x0fc,
  keycode_yacute              		= 0x0fd,
  keycode_thorn               		= 0x0fe,
  keycode_ydiaeresis          		= 0x0ff,

  keycode_shift,
  keycode_ctrl,
  keycode_alt,
  keycode_home,
  keycode_left,
  keycode_up,
  keycode_right,
  keycode_down,
  keycode_prior,
  keycode_page_up,
  keycode_next,
  keycode_page_down,
  keycode_end,
  keycode_begin,
  keycode_insert,
  keycode_f1,
  keycode_f2,
  keycode_f3,
  keycode_f4,
  keycode_f5,
  keycode_f6,
  keycode_f7,
  keycode_f8,
  keycode_f9,
  keycode_f10,
  keycode_f11,
  keycode_f12,

};

} // namespace infovis

#endif // INFOVIS_DRAWING_KEYCODES_HPP
